#include <QSqlDatabase>
#include <QFont>
#include <QMessageBox>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "datastore.h"
#include "logindialog.cpp"
#include "testselectdialog.cpp"
#include "commonfunctions.cpp"



MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindowClass)
{
    ui->setupUi(this);
    //skryti vsech prvku na zacatek
    ManipulateControlsTest(false);
    ui->tabWidget->setTabEnabled(1,false);
    ui->tabWidget->setTabEnabled(2,false);
    ui->lRes1->setVisible(false);
    ui->lRes2->setVisible(false);
    ui->lRes3->setVisible(false);
    ui->lRes4->setVisible(false);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_bNext_clicked()
{
    NextQuestion();
}
//pocatecni podpora pro prepinani modu
void MainWindow::on_action_Teacher_activated()
{
    if(Teacher) {
        ui->action_Insert_Mode->setVisible(true);
        ui->action_Teacher->setChecked(true);
        ui->action_User->setChecked(false);
        ui->tabWidget->setTabEnabled(1,true);
        ui->tabWidget->setTabEnabled(2,true);
    } else {
        QMessageBox msgBox;
        msgBox.setText("Please log in with teacher's privileges.");
        msgBox.exec();
        if (!KartellDB.isOpen()) {
            LoginDialog LoginWindow;
            LoginWindow.exec();
        }
        TestSelectDialog SelectWindow;
        SelectWindow.exec();
        ui->action_Teacher->setChecked(false);
    }
}

void MainWindow::on_action_User_activated()
{
    ui->action_Insert_Mode->setVisible(false);
    ui->action_User->setChecked(true);
    ui->action_Teacher->setChecked(false);
}

void MainWindow::on_action_Connect_activated()
{
    if (!KartellDB.isOpen()) {
        LoginDialog LoginWindow;
        LoginWindow.exec();
    }
    TestSelectDialog SelectWindow;
    SelectWindow.exec();
    LoadQuestions();
    if (Teacher) {
        ui->tabWidget->setTabEnabled(1,true);
        ui->tabWidget->setTabEnabled(2,true);
    }
}

//funkce pro skryti/odkryti ovladacich prvku
void MainWindow::ManipulateControlsTest(bool action)
{
    ui->lQuestion->setVisible(action);
    ui->bFinish->setVisible(action);
    ui->bNext->setVisible(action);
    ui->bPrevious->setVisible(action);
    ui->chAnswer1->setVisible(action);
    ui->chAnswer2->setVisible(action);
    ui->chAnswer3->setVisible(action);
    ui->chAnswer4->setVisible(action);
    ui->progressBar->setVisible(action);
}

void MainWindow::LoadQuestions()
{
    QSqlQuery query;
        query.prepare( "SELECT question_number,question,answer1,answer2,answer3,answer4,correct_answer,points,test_name FROM Testy WHERE test_name = ?" );
        query.bindValue( 0, TestName );
        query.exec();
        
    Questions = new sQuestion;

    //prvn zaznam v linearnim seznamu musi mit ukazatel Previous nastaveny na 0
    Questions->Previous= 0;
    //---------------------------------------------------------------------------
    TotalPoints = 0;

    while ( query.next() ){
       Questions->QuestionNumber = query.value(0).toInt();
       Questions->Question.append(query.value(1).toString());
       Questions->Answer1.append(query.value(2).toString());
       Questions->Answer2.append(query.value(3).toString());
       Questions->Answer3.append(query.value(4).toString());
       Questions->Answer4.append(query.value(5).toString());
       Questions->CorrectAnswer = query.value(6).toInt();
       Questions->UserAnswer = 0;
       TotalPoints += Questions->Points = query.value(7).toDouble();
       Questions->Next = new sQuestion;
       Questions->Next->Previous = Questions;
       Questions = Questions->Next;
       ui->progressBar->setMaximum(ui->progressBar->maximum()+1);
    }
    Questions = Questions->Previous;
    delete Questions->Next;
    Questions->Next = 0;
    query.finish();
    ManipulateControlsTest(true);
    while (Questions->Previous !=0) PreviousQuestion();
    ui->progressBar->setValue(1);
}

void MainWindow::ReloadQuestion()
{
    ui->lQuestion->setText(Questions->Question);
    ui->chAnswer1->setText(Questions->Answer1);
    ui->chAnswer2->setText(Questions->Answer2);
    ui->chAnswer3->setText(Questions->Answer3);
    ui->chAnswer4->setText(Questions->Answer4);
    LoadAnswers();
}


void MainWindow::NextQuestion()
{
    SaveAnswers();
    Questions = Questions->Next;
    ReloadQuestion();
    ui->progressBar->setValue(ui->progressBar->value()+1);
    if (Questions->Next == 0) {
        ui->bNext->setVisible(false);
        ui->bFinish->setVisible(true);
    }
    ui->bPrevious->setVisible(true);
    if(Finished) CompareResults();
}

void MainWindow::PreviousQuestion()
{
    SaveAnswers();
    Questions = Questions->Previous;
    ReloadQuestion();
    ui->progressBar->setValue(ui->progressBar->value()-1);
    if (Questions->Previous == 0) ui->bPrevious->setVisible(false);
    ui->bNext->setVisible(true);
    ui->bFinish->setVisible(false);
    if(Finished) CompareResults();
}

void MainWindow::on_bPrevious_clicked()
{
    PreviousQuestion();
}

void MainWindow::SaveAnswers()
{
    Questions->UserAnswer = 0;
    Questions->UserAnswer += ui->chAnswer1->isChecked() * 1;
    Questions->UserAnswer += ui->chAnswer2->isChecked() * 2;
    Questions->UserAnswer += ui->chAnswer3->isChecked() * 4;
    Questions->UserAnswer += ui->chAnswer4->isChecked() * 8;
}

void MainWindow::LoadAnswers()
{
    ui->chAnswer1->setChecked(Questions->UserAnswer %2);
    ui->chAnswer2->setChecked((Questions->UserAnswer / 2 == 1) | (Questions->UserAnswer / 2 == 3) | (Questions->UserAnswer / 2 == 5) | (Questions->UserAnswer / 2 == 7));
    ui->chAnswer3->setChecked((Questions->UserAnswer / 4 == 1) | (Questions->UserAnswer / 4 == 3));
    ui->chAnswer4->setChecked(Questions->UserAnswer / 8 == 1);
}

void MainWindow::ChangeControlsAbility(bool action)
{
    ui->chAnswer1->setEnabled(action);
    ui->chAnswer2->setEnabled(action);
    ui->chAnswer3->setEnabled(action);
    ui->chAnswer4->setEnabled(action);
}

void MainWindow::CompareResults()
{
    ui->lRes1->setVisible(Questions->UserAnswer %2 != Questions->CorrectAnswer %2);
    ui->lRes2->setVisible(((Questions->UserAnswer / 2 == 1) | (Questions->UserAnswer / 2 == 3) | (Questions->UserAnswer / 2 == 5) | (Questions->UserAnswer / 2 == 7))
                          != ((Questions->CorrectAnswer / 2 == 1) | (Questions->CorrectAnswer / 2 == 3) | (Questions->CorrectAnswer / 2 == 5) | (Questions->CorrectAnswer / 2 == 7)));
    ui->lRes3->setVisible(((Questions->UserAnswer / 4 == 1) | (Questions->UserAnswer / 4 == 3))
                          != ((Questions->CorrectAnswer / 4 == 1) | (Questions->CorrectAnswer/ 4 == 3)));
    ui->lRes4->setVisible((Questions->UserAnswer / 8 == 1) != (Questions->CorrectAnswer / 8 == 1));
}

void MainWindow::on_bFinish_clicked()
{
    if (!Finished)
    {
        SaveAnswers();
        ChangeControlsAbility(false);
        Finished = true;
        GenerateResultSet();
        SaveResultToSql();
        while (Questions->Previous !=0) PreviousQuestion();
    }
}

void MainWindow::on_tabWidget_currentChanged(int index)
{
    if (index == 2)
    {
        QSqlQuery query("SELECT test_name FROM Testy GROUP BY test_name");
        while( query.next()) {
            ui->cbTestSelect->addItem(query.value(0).toString(),query.value(0));
        }
        query.finish();
    }
}

void MainWindow::on_bClear_clicked()
{
    ui->leInsertQuestion->clear();
    ui->leInsertAnswer1->clear();
    ui->leInsertAnswer2->clear();
    ui->leInsertAnswer3->clear();
    ui->leInsertAnswer4->clear();
    ui->leInsertPoints->clear();
}

void MainWindow::on_bSave_clicked()
{
    if ( ui->leInsertQuestion->text().isEmpty() |
         ui->leInsertAnswer1->text().isEmpty()  |
         ui->leInsertAnswer2->text().isEmpty()  |
         ui->leInsertAnswer3->text().isEmpty()  |
         ui->leInsertAnswer4->text().isEmpty()  |
         ui->leInsertPoints->text().isEmpty()   |
         ui->cbTestSelect->currentText().isEmpty() )
    {
        QMessageBox msgBox;
        msgBox.setText("You haven't filled in all fields!");
        msgBox.exec();
    } else
    {
        QSqlQuery query;
        query.prepare( "INSERT INTO Testy (test_name, question_number, question, answer1, answer2, answer3, answer4, correct_answer, points) VALUES (?,?,?,?,?,?,?,?,?)");
        query.bindValue(0, ui->cbTestSelect->currentText());
        query.bindValue(1, (CountQuestions(ui->cbTestSelect->currentText())+1));
        query.bindValue(2, ui->leInsertQuestion->text());
        query.bindValue(3, ui->leInsertAnswer1->text());
        query.bindValue(4, ui->leInsertAnswer2->text());
        query.bindValue(5, ui->leInsertAnswer3->text());
        query.bindValue(6, ui->leInsertAnswer4->text());
        query.bindValue(7, (ui->chInsertAnswer1->isChecked()*1 +
                            ui->chInsertAnswer2->isChecked()*2 +
                            ui->chInsertAnswer3->isChecked()*4 +
                            ui->chInsertAnswer4->isChecked()*8));
        query.bindValue(8, ui->leInsertPoints->text().toInt());
        query.exec();
        query.finish();
    }
}
